import os

package_name = 'cythonbuilder'
console_name = 'cybuilder'
cython_build_dirname = 'build'
cython_extensions_dirname = 'ext'
cython_anno_dirname = os.path.join(cython_extensions_dirname, 'annotations')

